@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -74,7 +74,7 @@ namespace ipe {
 
   //! Ipelib version.
   /*! \ingroup base */
-  const int IPELIB_VERSION = 70010;
+  const int IPELIB_VERSION = 70014;
 
   //! Oldest readable file format version.
   /*! \ingroup base */
@@ -277,7 +277,7 @@ namespace ipe {
 
   class TellStream : public Stream {
   public:
-    virtual int tell() const = 0;
+    virtual long tell() const = 0;
   };
 
   class StringStream : public TellStream {
@@ -287,7 +287,7 @@ namespace ipe {
     virtual void putString(String s);
     virtual void putCString(const char *s);
     virtual void putRaw(const char *data, int size);
-    virtual int tell() const;
+    virtual long tell() const;
   private:
     String &iString;
   };
@@ -299,7 +299,7 @@ namespace ipe {
     virtual void putString(String s);
     virtual void putCString(const char *s);
     virtual void putRaw(const char *data, int size);
-    virtual int tell() const;
+    virtual long tell() const;
   private:
     std::FILE *iFile;
   };
