@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -138,8 +138,23 @@ void LayerBox::layerChanged(QListWidgetItem *item)
 void LayerBox::set(const Page *page, int view)
 {
   iInSet = true;
-  clear();
+  // find ordering of layers <first_view, layer>
+  std::vector<std::pair<int,int> > idx(page->countLayers());
   for (int i = 0; i < page->countLayers(); ++i) {
+    int first = page->countViews();
+    for (int j = 0; j < page->countViews(); ++j) {
+      if (page->visible(j, i)) {
+	first = j;
+	break;
+      }
+    }
+    idx[i] = std::pair<int,int>(first, i);
+  }
+  std::sort(idx.begin(), idx.end());
+
+  clear();
+  for (int j = 0; j < page->countLayers(); ++j) {
+    int i = idx[j].second;
     QListWidgetItem *item = new QListWidgetItem(QIpe(page->layer(i)), this);
     item->setFlags(Qt::ItemIsUserCheckable|Qt::ItemIsEnabled);
     item->setCheckState(page->visible(view, i) ? Qt::Checked : Qt::Unchecked);
@@ -201,8 +216,9 @@ void PathView::paintEvent(QPaintEvent *ev)
   cairo_fill(cc);
 
   if (iCascade) {
+    cairo_translate(cc, 0, h);
     double zoom = 2.0;
-    cairo_scale(cc, zoom, zoom);
+    cairo_scale(cc, zoom, -zoom);
     Vector v0 = (1.0/zoom) * Vector(0.1 * w, 0.5 * h);
     Vector v1 = (1.0/zoom) * Vector(0.7 * w, 0.5 * h);
     Vector u1 = (1.0/zoom) * Vector(0.88 * w, 0.8 * h);
@@ -331,6 +347,23 @@ void AppUi::addItem(QMenu *m, const QString &title, const char *name)
     a->setShortcut(QKeySequence(s));
     QString tt = title + " [" + s + "]";
     a->setToolTip(tt);
+  } else if (lua_istable(L, -1)) {
+    int no = lua_objlen(L, -1);
+    QList<QKeySequence> kl;
+    QString prim;
+    for (int i = 1; i <= no; ++i) {
+      lua_rawgeti(L, -1, i);
+      if (lua_isstring(L, -1)) {
+	QString s = lua_tostring(L, -1);
+	kl.append(QKeySequence(s));
+	if (prim.isEmpty())
+	  prim = s;
+      }
+      lua_pop(L, 1); // pop string
+    }
+    a->setShortcuts(kl);
+    QString tt = title + " [" + prim + "]";
+    a->setToolTip(tt);
   }
   a->setIcon(Prefs::get()->icon(name));
   lua_pop(L, 2);
@@ -362,6 +395,14 @@ void AppUi::addSnap(const char *name)
   iSnapTools->addAction(a);
 }
 
+void AppUi::addEdit(const char *name)
+{
+  QAction *a = findAction(name);
+  assert(a);
+  // a->setCheckable(true);
+  iEditTools->addAction(a);
+}
+
 void AppUi::buildMenus()
 {
   iMenu[EFileMenu] = menuBar()->addMenu("&File");
@@ -522,6 +563,7 @@ void AppUi::buildMenus()
   addItem(EZoomMenu, "Zoom out", "zoom_out");
   addItem(EZoomMenu, "Normal size", "normal_size");
   addItem(EZoomMenu, "Fit page", "fit_page");
+  addItem(EZoomMenu, "Fit width", "fit_width");
   addItem(EZoomMenu, "Fit objects", "fit_objects");
   addItem(EZoomMenu, "Fit selection", "fit_selection");
   iMenu[EZoomMenu]->addSeparator();
@@ -546,6 +588,7 @@ void AppUi::buildMenus()
   addItem(EViewMenu, "New view", "new_view");
   addItem(EViewMenu, "Delete view", "delete_view");
   iMenu[EViewMenu]->addSeparator();
+  addItem(EViewMenu, "Jump to view", "jump_view");
   addItem(EViewMenu, "Edit effects", "edit_effects");
 
   addItem(EPageMenu, "Next page", "next_page");
@@ -559,9 +602,14 @@ void AppUi::buildMenus()
   addItem(EPageMenu, "Paste page", "paste_page");
   addItem(EPageMenu, "Delete page", "delete_page");
   iMenu[EPageMenu]->addSeparator();
+  addItem(EPageMenu, "Jump to page", "jump_page");
   addItem(EPageMenu, "Edit title && sections", "edit_title");
+  addItem(EPageMenu, "Edit notes", "edit_notes");
+  addItem(EPageMenu, "Page sorter", "page_sorter");
+  iMenu[EPageMenu]->addSeparator();
 
   addItem(EHelpMenu, "Ipe &manual", "manual");
+  addItem(EHelpMenu, "Onscreen keyboard", "keyboard");
   addItem(EHelpMenu, "Show &configuration", "show_configuration");
   addItem(EHelpMenu, "About the &ipelets", "about_ipelets");
   addItem(EHelpMenu, "&About Ipe", "about");
@@ -620,6 +668,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   setCentralWidget(iCanvas);
 
   iSnapTools = addToolBar("Snap");
+  iEditTools = addToolBar("Edit");
   addToolBarBreak();
   iObjectTools = addToolBar("Objects");
 
@@ -658,6 +707,24 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   iSnapTools->addWidget(iSelector[EUiAngleSize]);
   addSnap("snapauto");
 
+  addEdit("copy");
+  addEdit("cut");
+  addEdit("paste");
+  addEdit("delete");
+  addEdit("undo");
+  addEdit("redo");
+  addEdit("zoom_in");
+  addEdit("zoom_out");
+  addEdit("fit_objects");
+  addEdit("fit_page");
+  addEdit("fit_width");
+  addEdit("keyboard");
+  iShiftKey = new QAction("shift_key", this);
+  iShiftKey->setCheckable(true);
+  iShiftKey->setIcon(Prefs::get()->icon("shift_key"));
+  iEditTools->addAction(iShiftKey);
+  connect(iShiftKey, SIGNAL(triggered()), SLOT(toolbarModifiersChanged()));
+
   findAction("fullscreen")->setCheckable(true);
   findAction("grid_visible")->setCheckable(true);
 
@@ -670,6 +737,13 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   addDockWidget(Qt::LeftDockWidgetArea, iLayerTools);
   iLayerTools->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
 
+  iNotesTools = new QDockWidget("Notes", this);
+  addDockWidget(Qt::RightDockWidgetArea, iNotesTools);
+  iNotesTools->setAllowedAreas(Qt::LeftDockWidgetArea|
+			       Qt::RightDockWidgetArea);
+  iMenu[EPageMenu]->addAction(iNotesTools->toggleViewAction());
+
+
   iBookmarkTools = new QDockWidget("Bookmarks", this);
   addDockWidget(Qt::RightDockWidgetArea, iBookmarkTools);
   iBookmarkTools->setAllowedAreas(Qt::LeftDockWidgetArea|
@@ -681,6 +755,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   iObjectTools->setObjectName(QLatin1String("ObjectTools"));
   iPropertiesTools->setObjectName(QLatin1String("PropertiesTools"));
   iLayerTools->setObjectName(QLatin1String("LayerTools"));
+  iNotesTools->setObjectName(QLatin1String("NotesTools"));
   iBookmarkTools->setObjectName(QLatin1String("BookmarkTools"));
 
   QWidget *wg = new QFrame();
@@ -752,14 +827,28 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   iViewNumber->setToolTip("Current view number");
   iPageNumber->setText("Page 1/1");
   iPageNumber->setToolTip("Current page number");
+  iViewMarked = new QCheckBox();
+  iPageMarked = new QCheckBox();
+  iViewMarked->setFocusPolicy(Qt::NoFocus);
+  iPageMarked->setFocusPolicy(Qt::NoFocus);
   bg->addButton(iViewNumber, EUiView);
   bg->addButton(iPageNumber, EUiPage);
+  bg->addButton(iViewMarked, EUiViewMarked);
+  bg->addButton(iPageMarked, EUiPageMarked);
   hol->setSpacing(0);
+  hol->addWidget(iViewMarked);
   hol->addWidget(iViewNumber);
   hol->addStretch(1);
+  hol->addWidget(iPageMarked);
   hol->addWidget(iPageNumber);
   lo->addLayout(hol, EUiSymbolSize + 2, 0, 1, -1);
 
+  iPageNotes = new QTextEdit();
+  iPageNotes->setAcceptRichText(false);
+  iPageNotes->setReadOnly(true);
+  iPageNotes->setFocusPolicy(Qt::NoFocus);
+  iNotesTools->setWidget(iPageNotes);
+
   iBookmarks = new QListWidget();
   iBookmarks->setToolTip("Bookmarks of this document");
   iBookmarks->setFocusPolicy(Qt::NoFocus);
@@ -788,11 +877,17 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
   findAction("coordinates|points")->setChecked(true);
   statusBar()->addPermanentWidget(iMouse, 0);
 
+  iCoordinatesFormat = "%g%s, %g%s";
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "coordinates_format");
+  if (lua_isstring(L, -1)) {
+    iCoordinatesFormat = lua_tostring(L, -1);
+  }
+  lua_pop(L, 2);
+
   iResolution = new QLabel(statusBar());
   statusBar()->addPermanentWidget(iResolution, 0);
 
-  iBookmarkTools->hide();
-
   connect(iCanvas, SIGNAL(wheelMoved(int)),   SLOT(wheelZoom(int)));
   connect(iCanvas, SIGNAL(mouseAction(int)),  SLOT(mouseAction(int)));
   connect(iCanvas, SIGNAL(positionChanged()), SLOT(positionChanged()));
@@ -895,7 +990,7 @@ void AppUi::positionChanged()
   adjust(v.y, iMouseIn);
   const char *units = mouse_units[iMouseIn];
   QString s;
-  s.sprintf("%g%s, %g%s", v.x, units, v.y, units);
+  s.sprintf(iCoordinatesFormat.z(), v.x, units, v.y, units);
   /* TODO: if tool active, show pos relative to origin
   if (!iFileTools->isEnabled()) {
     IpeVector u = pos - iMouseBase;
@@ -916,6 +1011,16 @@ void AppUi::toolChanged(bool hasTool)
   setActionsEnabled(!hasTool);
 }
 
+void AppUi::toolbarModifiersChanged()
+{
+  if (iCanvas) {
+    int mod = 0;
+    if (iShiftKey->isChecked())
+      mod |= Qt::ShiftModifier;
+    iCanvas->setAdditionalModifiers(mod);
+  }
+}
+
 // --------------------------------------------------------------------
 
 static String stripMark(Attribute mark)
@@ -1051,22 +1156,33 @@ void AppUi::setGridAngleSize(Attribute abs_grid, Attribute abs_angle)
   }
 }
 
-void AppUi::setNumbers(String vno, String pno)
+void AppUi::setNumbers(String vno, bool vm, String pno, bool pm)
 {
   if (vno.isEmpty()) {
     iViewNumber->hide();
+    iViewMarked->hide();
   } else {
     iViewNumber->setText(QIpe(vno));
     iViewNumber->show();
+    iViewMarked->setCheckState(vm ? Qt::Checked : Qt::Unchecked);
+    iViewMarked->show();
   }
   if (pno.isEmpty()) {
     iPageNumber->hide();
+    iPageMarked->hide();
   } else {
     iPageNumber->show();
+    iPageMarked->show();
     iPageNumber->setText(QIpe(pno));
+    iPageMarked->setCheckState(pm ? Qt::Checked : Qt::Unchecked);
   }
 }
 
+void AppUi::setNotes(String notes)
+{
+  iPageNotes->setPlainText(QIpe(notes));
+}
+
 void AppUi::setLayers(const Page *page, int view)
 {
   iLayerList->set(page, view);
@@ -1100,6 +1216,27 @@ void AppUi::bookmarkSelected(QListWidgetItem *item)
   lua_call(L, 2, 0);
 }
 
+int AppUi::showTool(lua_State *L)
+{
+  static const char * const option_names[] = {
+    "properties", "bookmarks", "notes", "layers", 0 };
+  int m = luaL_checkoption(L, 2, 0, option_names);
+  bool s = lua_toboolean(L, 3);
+  QWidget *tool = 0;
+  switch (m) {
+  case 0: tool = iPropertiesTools; break;
+  case 1: tool = iBookmarkTools; break;
+  case 2: tool = iNotesTools; break;
+  case 3: tool = iLayerTools; break;
+  default: break;
+  }
+  if (s)
+    tool->show();
+  else
+    tool->hide();
+  return 0;
+}
+
 void AppUi::setZoom(double zoom)
 {
   QString s;
@@ -1201,7 +1338,21 @@ void AppUi::callSelector(const char *name, double scalar)
 
 static const char * const selector_name[] =
   { "stroke", "fill", "pen", "textsize", "markshape",
-    "symbolsize", "gridsize", "anglesize", "view", "page" };
+    "symbolsize", "gridsize", "anglesize", "view", "page",
+    "viewmarked", "pagemarked" };
+
+bool AppUi::checkbox(lua_State *L)
+{
+  int id = luaL_checkoption(L, 2, 0, selector_name);
+  switch (id) {
+  case EUiViewMarked:
+    return (iViewMarked->checkState() == Qt::Checked);
+  case EUiPageMarked:
+    return (iPageMarked->checkState() == Qt::Checked);
+  default:
+    return false;
+  }
+}
 
 void AppUi::absoluteButton(int id)
 {
@@ -1237,7 +1388,7 @@ void AppUi::layerAction(String name, String layer)
 
 static const char * const aboutText =
 "<qt><h2>Ipe %d.%d.%d</h2>"
-"<p>Copyright (c) 1993-2009 Otfried Cheong</p>"
+"<p>Copyright (c) 1993-2010 Otfried Cheong</p>"
 "<p>The extensible drawing editor Ipe creates figures "
 "in Postscript and PDF format, "
 "using LaTeX to format the text in the figures.</p>"
@@ -1259,8 +1410,8 @@ static const char * const aboutText =
 void AppUi::aboutIpe()
 {
   int luaSize = lua_gc(L, LUA_GCCOUNT, 0);
-  char buf[strlen(aboutText) + 100];
-  sprintf(buf, aboutText,
+  std::vector<char> buf(strlen(aboutText) + 100);
+  sprintf(&buf[0], aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100,
@@ -1273,7 +1424,7 @@ void AppUi::aboutIpe()
   QMessageBox msgBox(this);
   msgBox.setWindowTitle("About Ipe");
   msgBox.setWindowIcon(Prefs::get()->icon("ipe"));
-  msgBox.setInformativeText(buf);
+  msgBox.setInformativeText(&buf[0]);
   msgBox.setIconPixmap(Prefs::get()->pixmap("ipe"));
   msgBox.setStandardButtons(QMessageBox::Ok);
   msgBox.exec();
