@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 
 #include <QMainWindow>
 #include <QAction>
+#include <QCheckBox>
 #include <QToolBar>
 #include <QDockWidget>
 #include <QActionGroup>
@@ -43,6 +44,7 @@
 #include <QLabel>
 #include <QComboBox>
 #include <QToolButton>
+#include <QTextEdit>
 
 using namespace ipe;
 using namespace ipeqt;
@@ -137,7 +139,7 @@ public:
   // change list in front of AppUi::selector if enum changes
   enum { EUiStroke, EUiFill, EUiPen, EUiTextSize, EUiMarkShape,
 	 EUiSymbolSize, EUiGridSize, EUiAngleSize, EUiNum,
-	 EUiView = EUiNum, EUiPage };
+	 EUiView = EUiNum, EUiPage, EUiViewMarked, EUiPageMarked };
 
 public:
   AppUi(lua_State *L0, int model, Qt::WFlags f=0);
@@ -151,10 +153,13 @@ public:
   void setZoom(double zoom);
   void setActionsEnabled(bool mode);
   QAction *findAction(const char *name) const;
-  void setNumbers(String vno, String pno);
+  void setNumbers(String vno, bool vm, String pno, bool pm);
+  void setNotes(String notes);
+  bool checkbox(lua_State *L);
 
   // direct Lua methods
   int setBookmarks(lua_State *L);
+  int showTool(lua_State *L);
 
 public slots:
   void action(String name);
@@ -167,6 +172,7 @@ public slots:
   void mouseAction(int button);
   void positionChanged();
   void toolChanged(bool hasTool);
+  void toolbarModifiersChanged();
   void aboutIpe();
 
   void wheelZoom(int degrees);
@@ -186,6 +192,7 @@ private:
   void addItem(QMenu *m, const QString &title, const char *name);
   void addItem(int m, const QString &title, const char *name);
   void addSnap(const char *name);
+  void addEdit(const char *name);
   void buildMenus();
   void callSelector(String name, String value);
   void callSelector(const char *name, Color color);
@@ -216,18 +223,27 @@ private:
   QToolButton *iViewNumber;
   QToolButton *iPageNumber;
 
+  QCheckBox *iViewMarked;
+  QCheckBox *iPageMarked;
+
   QToolBar *iSnapTools;
+  QToolBar *iEditTools;
   QToolBar *iObjectTools;
 
   QDockWidget *iPropertiesTools;
   QDockWidget *iLayerTools;
   QDockWidget *iBookmarkTools;
+  QDockWidget *iNotesTools;
 
   QActionGroup *iModeActionGroup;
 
+  QAction *iShiftKey;
+
   QListWidget *iBookmarks;
   LayerBox *iLayerList;
+  QTextEdit *iPageNotes;
 
+  String iCoordinatesFormat;
   int iMouseIn;
   QLabel *iMouse;
   QLabel *iResolution;
