@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 --]]
 
 -- main entry point to all actions
--- (except layeraction, mouseaction, selector)
+-- (except mouseaction, selector)
 
 -- protects call, then distributes to action_xxx method or
 -- saction_xxx methods (the latter are only called if a selection exists)
@@ -167,25 +167,66 @@ function MODEL:absoluteButton(button)
       self:set_absolute(button, d)
     end
   elseif button == "view" then
-    local d = ipeui.getInt(self.ui, "Select view", "View number:",
-			   self.vno, 1, self:page():countViews(), 1)
-    if d then
-      self.vno = d
-      self:setPage()
-    end
+    self:action_jump_view()
   elseif button == "page" then
-    local d = ipeui.getInt(self.ui, "Select page", "Page number:",
-			   self.pno, 1, #self.doc, 1)
-    if d then
-      self.pno = d
-      self.vno = 1
-      self:setPage()
-    end
+    self:action_jump_page()
+  elseif button == "viewmarked" then
+    self:action_mark_view(self.ui:checkbox(button))
+  elseif button == "pagemarked" then
+    self:action_mark_page(self.ui:checkbox(button))
   else
     print("Unknown button: ", button)
   end
 end
 
+function MODEL:action_jump_view()
+  local d = self.ui:selectPage(self.doc, self.pno, self.vno)
+  if d then
+    self.vno = d
+    self:setPage()
+  end
+end
+
+function MODEL:action_jump_page()
+  local d = self.ui:selectPage(self.doc, nil, self.pno)
+  if d then
+    self.pno = d
+    self.vno = 1
+    self:setPage()
+  end
+end
+
+function MODEL:action_mark_view(m)
+  local t = { label="set view mark to " .. tostring(m),
+	      pno=self.pno,
+	      vno=self.vno,
+	      original=self:page():markedView(self.vno),
+	      final=m,
+	    }
+  t.undo = function (t, doc)
+	     doc[t.pno]:setMarkedView(t.vno, t.original)
+	   end
+  t.redo = function (t, doc)
+	     doc[t.pno]:setMarkedView(t.vno, t.final)
+	   end
+  self:register(t)
+end
+
+function MODEL:action_mark_page(m)
+  local t = { label="set page mark to " .. tostring(m),
+	      pno=self.pno,
+	      original=self:page():marked(),
+	      final=m,
+	    }
+  t.undo = function (t, doc)
+	     doc[t.pno]:setMarked(t.original)
+	   end
+  t.redo = function (t, doc)
+	     doc[t.pno]:setMarked(t.final)
+	   end
+  self:register(t)
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:showPathStylePopup(v)
@@ -289,6 +330,7 @@ function MODEL:layeraction_select(layer, arg)
 	     doc[t.pno]:setVisible(t.vno, t.layer, t.visible)
 	   end
   self:register(t)
+  self:deselectNotInView()
 end
 
 function MODEL:layeraction_lock(layer, arg)
@@ -532,6 +574,16 @@ function MODEL:action_about_ipelets()
   ipeui.messageBox(self.ui, "information", "About the ipelets", s)
 end
 
+function MODEL:action_keyboard()
+  local s = prefs.keyboard
+  if s then
+    os.execute(s)
+  else
+    self:warning("No onscreen keyboard defined.",
+		 "Edit preferences to define an onscreen keyboard.")
+  end
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:action_set_origin_snap()
@@ -795,27 +847,36 @@ end
 function MODEL:action_fit_page()
   local layout = self.doc:sheets():find("layout")
   local box = ipe.Rect()
-  box:add(-layout.origin)
-  box:add(-layout.origin + layout.papersize)
+  box:add(-layout.origin - V(2,2))
+  box:add(-layout.origin + layout.papersize + V(2,2))
+  self:fitBox(box);
+end
+
+function MODEL:action_fit_width()
+  local layout = self.doc:sheets():find("layout")
+  local box = ipe.Rect()
+  local y0 = self.ui:pan().y
+  box:add(V(-layout.origin.x - 2, y0 - 2))
+  box:add(V(-layout.origin.x + layout.papersize.x + 2, y0 + 2))
   self:fitBox(box);
 end
 
 function MODEL:action_fit_objects()
   local box = ipe.Rect()
+  local m = ipe.Matrix()
   local p = self:page()
   for i,obj,_,layer in p:objects() do
-    if p:visible(self.vno, i) then
-      box:add(p:bbox(i))
-    end
+    if p:visible(self.vno, i) then obj:addToBBox(box, m, false) end
   end
   self:fitBox(box);
 end
 
 function MODEL:saction_fit_selection()
   local box = ipe.Rect()
+  local m = ipe.Matrix()
   local p = self:page()
   for i,obj,sel,_ in p:objects() do
-    if sel then box:add(p:bbox(i)) end
+    if sel then obj:addToBBox(box, m, false) end
   end
   self:fitBox(box);
 end
@@ -1125,6 +1186,10 @@ function MODEL:action_edit_title()
   if not d:get("tsubsection") then
     final.subsection = d:get("subsection")
   end
+  if prefs.automatic_use_title and ti.title ~= final.title then
+    final.section = true
+    final.tsection = true
+  end
   local t = { label="change title and sections of page " .. self.pno,
 	      pno = self.pno,
 	      vno = self.vno,
@@ -1140,6 +1205,87 @@ function MODEL:action_edit_title()
   self:register(t)
 end
 
+function MODEL:action_edit_notes()
+  local d = ipeui.Dialog(self.ui, "Ipe: Edit page notes")
+  d:add("label1", "label", { label="<b>Page notes</b>"}, 1, 1, 1, 4)
+  d:add("notes", "text", {}, 2, 1, 1, 4)
+  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 3)
+  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 4)
+  d:setStretch("row", 2, 1)
+  d:setStretch("column", 2, 1)
+  -- setup original values
+  local n = self:page():notes()
+  d:set("notes", n)
+  if not d:execute() then return end
+  local final = d:get("notes")
+  if string.match(final, "^%s*$") then
+    final = ""
+  end
+  local t = { label="change notes on page " .. self.pno,
+	      pno = self.pno,
+	      vno = self.vno,
+	      original = n,
+	      final = final,
+	    }
+  t.undo = function (t, doc)
+	     doc[t.pno]:setNotes(t.original)
+	   end
+  t.redo = function (t, doc)
+	     doc[t.pno]:setNotes(t.final)
+	   end
+  self:register(t)
+end
+
+-- rearrange pages of document
+-- new order is given by arr
+-- 0 entries in arr are taken from newpages
+-- returns list of pages that remained unused
+function arrange_pages(doc, arr, newpages)
+  -- take all pages from document
+  local pg = {}
+  while #doc > 0 do
+    pg[#pg+1] = doc:remove(1)
+  end
+  -- insert pages again in new order
+  for i = 1,#arr do
+    local no = arr[i]
+    if no > 0 then
+      doc:append(pg[no])  -- clones the page
+      pg[no] = nil        -- mark as used
+    else
+      doc:append(table.remove(newpages, 1))
+    end
+  end
+  -- return unused pages
+  local unused = {}
+  for i = 1,#pg do
+    if pg[i] then unused[#unused+1] = pg[i] end
+  end
+  return unused
+end
+
+function MODEL:action_page_sorter()
+  local arr = self.ui:pageSorter(self.doc)
+  if not arr then return end -- canceled
+  -- compute reverse 'permutation'
+  local rev = {}
+  for i = 1,#self.doc do rev[i] = 0 end
+  for i = 1,#arr do rev[arr[i]] = i end
+  local t = { label="rearrangement of pages",
+	      pno = 1,
+	      vno = 1,
+	      arr = arr,
+	      rev = rev,
+	    }
+  t.undo = function (t, doc)
+	     arrange_pages(doc, t.rev, t.deleted)
+	   end
+  t.redo = function (t, doc)
+	     t.deleted = arrange_pages(doc, t.arr, {})
+	   end
+  self:register(t)
+end
+
 ----------------------------------------------------------------------
 
 function MODEL:action_new_layer()
@@ -1840,9 +1986,10 @@ function MODEL:saction_add_clipping()
 	      original = obj:clone(),
 	      path = p[sec]:clone(),
 	      shape = shape,
+	      layer = p:layerOf(sec)
 	    }
   t.undo = function (t, doc)
-	     doc[t.pno]:insert(t.secondary, t.path)
+	     doc[t.pno]:insert(t.secondary, t.path, nil, t.layer)
 	     doc[t.pno]:replace(t.primary, t.original)
 	   end
   t.redo = function (t, doc)
