@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -161,6 +161,22 @@ local function usage()
   io.stderr:write("or:    ipe --help\n")
 end
 
+-- check locale
+test1 = string.format("%g", 1.5)
+test2 = string.format("%s", tonumber("1.5"))
+if test1 ~= "1.5" or test2 ~= "1.5" then
+   m = "<qt>Formatting the number <code>1.5</code> results in '"
+      .. test1 .. "'. "
+      .. "Reading '1.5' results in <code>" .. test2 .. "</code><br />"
+      .. "Therefore Ipe will not work correctly when loading or saving files. "
+      .. "<em>Please report this problem.</em><br />"
+      .. "As a workaround, you can start Ipe from the commandline like this: "
+      .. "<pre>export LANG=C\nexport LC_NUMERIC=C\nipe</pre></qt>"
+   ipeui.messageBox(nil, "critical",
+		    "Ipe is running with an incorrect locale", m)
+   return
+end
+
 config.ipeletDirs = {}
 for w in string.gmatch(config.ipelets, "[^;]+") do
   config.ipeletDirs[#config.ipeletDirs + 1] = w
@@ -170,41 +186,6 @@ if config.platform == "unix" then
   table.insert(config.ipeletDirs, 1, config.home .. "/.ipe/ipelets")
 end
 
-if #argv == 1 and argv[1] == "-show-configuration" then
-  show_configuration()
-  return
-end
-
-if #argv == 1 and (argv[1] == "--help" or argv[1] == "-h") then
-  usage()
-  return
-end
-
-local first_file = nil
-local i = 1
-local style_sheets = {}
-
-while i <= #argv do
-  if argv[i] == "-sheet" then
-    if i == #argv then usage() return end
-    style_sheets[#style_sheets + 1] = argv[i+1]
-    i = i + 2
-  else
-    if i ~= #argv then usage() return end
-    first_file = argv[i]
-    i = i + 1
-  end
-end
-
-if #style_sheets > 0 then prefs.styles = style_sheets end
-
-config.styleList = {}
-for _,w in ipairs(prefs.styles) do
-  if w:sub(-4) ~= ".isy" then w = w .. ".isy" end
-  if not w:find("/") then w = config.styles .. "/" .. w end
-  config.styleList[#config.styleList + 1] = w
-end
-
 -- look for ipelets
 ipelets = {}
 for _,w in ipairs(config.ipeletDirs) do
@@ -240,6 +221,41 @@ for _,w in ipairs(config.ipeletDirs) do
   end
 end
 
+if #argv == 1 and argv[1] == "-show-configuration" then
+  show_configuration()
+  return
+end
+
+if #argv == 1 and (argv[1] == "--help" or argv[1] == "-h") then
+  usage()
+  return
+end
+
+local first_file = nil
+local i = 1
+local style_sheets = {}
+
+while i <= #argv do
+  if argv[i] == "-sheet" then
+    if i == #argv then usage() return end
+    style_sheets[#style_sheets + 1] = argv[i+1]
+    i = i + 2
+  else
+    if i ~= #argv then usage() return end
+    first_file = argv[i]
+    i = i + 1
+  end
+end
+
+if #style_sheets > 0 then prefs.styles = style_sheets end
+
+config.styleList = {}
+for _,w in ipairs(prefs.styles) do
+  if w:sub(-4) ~= ".isy" then w = w .. ".isy" end
+  if not w:find("/") then w = config.styles .. "/" .. w end
+  config.styleList[#config.styleList + 1] = w
+end
+
 MODEL:new(first_file)
 
 ipeui.mainLoop()
