@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -103,6 +103,13 @@ function MODEL:init(fname)
   self.ui:setActionState("mode_select", true)
   self.ui:setActionState("grid_visible", self.snap.grid_visible)
 
+  if #self:getBookmarks() == 0 then
+    self.ui:showTool("bookmarks", false)
+  end
+  if self:page():notes() == "" then
+    self.ui:showTool("notes", false)
+  end
+
   if prefs.autosave_interval then
     self.timer = ipeui.Timer(self, "autosave")
     self.timer:setInterval(1000 * prefs.autosave_interval) -- millisecs
@@ -193,23 +200,26 @@ end
 
 -- Set canvas to current page
 function MODEL:setPage()
-  self.ui:setPage(self.doc[self.pno], self.vno, self.doc:sheets())
-  self.ui:setLayers(self:page(), self.vno)
+  local p = self:page()
+  self.ui:setPage(p, self.pno, self.vno, self.doc:sheets())
+  self.ui:setLayers(p, self.vno)
+  self.ui:setNumbering(self.doc:properties().numberpages)
   self.ui:update()
   self:setCaption()
   self:setBookmarks()
+  self.ui:setNotes(p:notes())
   local vno
-  if self:page():countViews() > 1 then
-    vno = string.format("View %d/%d", self.vno, self:page():countViews())
+  if p:countViews() > 1 then
+    vno = string.format("View %d/%d", self.vno, p:countViews())
   end
   local pno
   if #self.doc > 1 then
     pno = string.format("Page %d/%d", self.pno, #self.doc)
   end
-  self.ui:setNumbers(vno, pno)
+  self.ui:setNumbers(vno, p:markedView(self.vno), pno, p:marked())
 end
 
-function MODEL:setBookmarks()
+function MODEL:getBookmarks()
   local b = {}
   for _,p in self.doc:pages() do
     local t = p:titles()
@@ -220,7 +230,11 @@ function MODEL:setBookmarks()
       if t.subsection ~= "" then b[#b+1] = t.subsection end
     elseif t.title ~= "" then b[#b+1] = "   " .. t.title end
   end
-  self.ui:setBookmarks(b)
+  return b
+end
+
+function MODEL:setBookmarks()
+  self.ui:setBookmarks(self:getBookmarks())
 end
 
 function MODEL:findPageForBookmark(index)
@@ -268,7 +282,7 @@ end
 -- If primaryOnly is true, the primary selection has to be close enough,
 -- otherwise it'll be replaced as above.
 function MODEL:updateCloseSelection(primaryOnly)
-  local bound = prefs.select_distance
+  local bound = prefs.close_distance
   local pos = self.ui:unsnappedPos()
   local p = self:page()
 
@@ -285,10 +299,10 @@ function MODEL:updateCloseSelection(primaryOnly)
   -- current selection is not close enough: find closest object
   local closest
   for i,obj,sel,layer in p:objects() do
-    if (p:visible(self.vno, i) and not p:isLocked(layer)) then
-      local d = p:distance(i, pos, bound)
-      if d < bound then closest = i; bound = d end
-    end
+     if p:visible(self.vno, i) and not p:isLocked(layer) then
+	local d = p:distance(i, pos, bound)
+	if d < bound then closest = i; bound = d end
+     end
   end
 
   if closest then
