@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -48,28 +48,46 @@ prefs.autosave_filename = config.home .. "/%s.autosave"
 -- External editor for editing text objects
 -- must contain '%s' for the temporary filename
 -- set this to nil to hide the "Editor" button on text dialogs
+-- by default this is disabled on Windows as a
+-- DOS window will pop up when calling the editor
 if os.getenv("EDITOR") then
   prefs.external_editor = os.getenv("EDITOR") .. " %s"
-else
+elseif config.platform ~= "win" then
   prefs.external_editor = "gedit %s"
   -- prefs.external_editor = "emacsclient %s"
+else
+  prefs.external_editor = nil
 end
 
+-- Should the external editor be called automatically?
+prefs.auto_external_editor = nil
+
 -- Size of dialog window for creating/editing text objects
 prefs.editor_size = { 600, 400 }
 
 -- Size of main window at startup
-prefs.window_size = { 800, 600 }
+prefs.window_size = { 960, 600 }
+
+-- Size of page sorter window
+prefs.page_sorter_size = { 960, 600 }
+
+-- Width of page thumbnails (height is computed automatically)
+prefs.thumbnail_width = 300
 
 -- Canvas customization:
-prefs.paper_color = { r = 1.0, g = 1.0, b = 1.0 }  -- white
--- prefs.paper_color = { r = 1.0, g = 1.0, b = 0.5 }  -- classic Ipe 6 yellow
--- classic grid uses dots instead of lines
-prefs.classic_grid = false
--- line width of grid lines
--- if classic_grid is true, then thin_grid_lines is size of grid dots
-prefs.thin_grid_line = 0.1
-prefs.thick_grid_line = 0.3
+prefs.canvas_style = {
+  paper_color = { r = 1.0, g = 1.0, b = 1.0 },  -- white
+  -- paper_color = { r = 1.0, g = 1.0, b = 0.5 }  -- classic Ipe 6 yellow
+  -- classic grid uses dots instead of lines
+  classic_grid = false,
+  -- line width of grid lines
+  -- if classic_grid is true, then thin_grid_lines is size of grid dots
+  thin_grid_line = 0.1,
+  thick_grid_line = 0.3,
+  -- steps indicate multiples of grid distance where grid lines are drawn
+  thin_step = 1, thick_step = 4,
+  -- e.g. try this: thin_step = 2, thick_step = 5
+}
 
 -- Should the grid be visible when Ipe starts? (true or false)
 prefs.grid_visible = true
@@ -81,6 +99,9 @@ prefs.angle_size = 45    -- degrees
 -- Maximum distance in pixels selecting/snapping
 prefs.select_distance = 36
 prefs.snap_distance = 16
+-- When transforming objects, if currently select object is further than
+-- this distance, the closest object is selected instead
+prefs.close_distance = 48
 
 -- Minimal and maximal possible zoom
 prefs.min_zoom = 0.1
@@ -89,6 +110,10 @@ prefs.max_zoom = 100
 -- Should newly created text be transformable by default?
 prefs.text_transformable = false
 
+-- If set to true, then whenever the user edits the title of a page,
+-- the check box "section: use title" is checked automatically.
+prefs.automatic_use_title = false
+
 -- How to start browser to show Ipe manual
 if config.platform == "apple" then
   prefs.browser = "open %s"
@@ -98,6 +123,17 @@ else
   prefs.browser = "gnome-open %s"
 end
 
+-- How to start onscreen keyboard
+if config.platform == "unix" then
+  prefs.keyboard = "onboard &"
+else
+  prefs.keyboard = nil
+end
+
+-- format string for the coordinates in the status bar
+-- (x, unit, y, unit)
+prefs.coordinates_format = "%g%s, %g%s"
+
 -- Auto-exporting when document is being saved
 -- if auto_export_only_if_exists is true, then the file will only
 -- be auto-exported if a file with the target name already exists
