@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ function externalEditor(d, field)
 end
 
 function addEditorField(d, field, row, col)
-  if config.platform ~= "win" and prefs.external_editor then
+  if prefs.external_editor then
     d:add("editor", "button",
 	  { label="&Editor",
 	    action=function (d) externalEditor(d, field) end },
@@ -236,18 +236,19 @@ function LINESTOOL:explain()
   end
   s = s .. " | Right: final vtx"
   if self:has_segs(2) then
-    s = s .. " | S: spline mode"
+    s = s .. " | " .. shortcuts_linestool.spline .. ": spline mode"
   end
   if self:has_segs(3) then
-    s = s .. " | Q: quadratic Bezier | A: circle arc"
+    s = s .. " | " .. shortcuts_linestool.quadratic_bezier .. ": quadratic Bezier"
+    s = s .. " | " .. shortcuts_linestool.arc .. ": circle arc"
   end
   if self:has_segs(4) then
-    s = s .. " | C: cubic Bezier"
+    s = s .. " | " .. shortcuts_linestool.cubic_bezier ..": cubic Bezier"
   end
   if #self.v > 2 and self.t[#self.t - 1] == VERTEX then
-    s = s .. " | Y: set axis"
+    s = s .. " | " .. shortcuts_linestool.set_axis ..": set axis"
   end
-  self.model.ui:explain(s)
+  self.model.ui:explain(s, 0)
 end
 
 function LINESTOOL:key(code, modifiers, text)
@@ -272,28 +273,29 @@ function LINESTOOL:key(code, modifiers, text)
   elseif text == "\027" then
     self.model.ui:finishTool()
     return true
-  elseif self:has_segs(2) and text == "s" then
+  elseif self:has_segs(2) and text == shortcuts_linestool.spline then
     self.t[#self.t] = SPLINE
     self:compute()
     self.model.ui:update(false)
     self:explain()
-  elseif self:has_segs(3) and text == "q" then
+  elseif self:has_segs(3) and text == shortcuts_linestool.quadratic_bezier then
     self.t[#self.t - 1] = QUAD
     self:compute()
     self.model.ui:update(false)
     self:explain()
-  elseif self:has_segs(3) and text == "a" then
+  elseif self:has_segs(3) and text == shortcuts_linestool.arc then
     self.t[#self.t - 1] = ARC
     self:compute()
     self.model.ui:update(false)
     self:explain()
-  elseif self:has_segs(4) and text == "c" then
+  elseif self:has_segs(4) and text == shortcuts_linestool.cubic_bezier then
     self.t[#self.t - 1] = BEZIER
     self.t[#self.t - 2] = BEZIER
     self:compute()
     self.model.ui:update(false)
     self:explain()
-  elseif #self.v > 2 and self.t[#self.t - 1] == VERTEX and text == "y" then
+  elseif #self.v > 2 and self.t[#self.t - 1] == VERTEX and
+    text == shortcuts_linestool.set_axis then
     -- set axis
     self.model.snap.with_axes = true
     self.model.snap.snapangle = true
@@ -387,7 +389,7 @@ end
 
 function SPLINEGONTOOL:explain()
   local s = "Left: Add vertex | Right: Add final vertex | Del: Delete vertex"
-  self.model.ui:explain(s)
+  self.model.ui:explain(s, 0)
 end
 
 function SPLINEGONTOOL:mouseButton(button, modifiers, press)
@@ -609,7 +611,10 @@ function INKTOOL:new(model)
   local v = model.ui:pos()
   tool.v = { v }
   model.ui:shapeTool(tool)
-  tool.setColor(1.0, 0, 0)
+  local s = model.doc:sheets():find("color", model.attributes.stroke)
+  tool.setColor(s.r, s.g, s.b)
+  local w = model.doc:sheets():find("pen", model.attributes.pen)
+  model.ui:setCursor(w, s.r, s.g, s.b)
   return tool
 end
 
@@ -624,8 +629,18 @@ end
 function INKTOOL:mouseButton(button, modifiers, press)
   self.model.ui:finishTool()
   if self.shape then
-    local obj = ipe.Path(self.model.attributes, { self.shape })
-    self.model:creation("create ink path", obj)
+     local obj = ipe.Path(self.model.attributes, { self.shape })
+     -- round linecaps are prettier for handwriting
+     obj:set("linecap", "round")
+     self.model:creation("create ink path", obj)
+  else
+     -- mouse was pressed and released without movement
+     local shape = { type="curve", closed=false,
+		     { type="segment", self.v[1], self.v[1] } }
+     local obj = ipe.Path(self.model.attributes, { shape })
+     -- must make round linecap, otherwise it will be invisible
+     obj:set("linecap", "round")
+     self.model:creation("create ink dot", obj)
   end
 end
 
@@ -670,6 +685,10 @@ function MODEL:createText(mode)
   addEditorField(d, "text", 3, 2)
   d:setStretch("row", 2, 1)
   d:setStretch("column", 1, 1)
+  d:set("ignore-cancel", true)
+  if prefs.auto_external_editor then
+    externalEditor(d, "text")
+  end
   local r = d:execute(prefs.editor_size)
   if r then
     local t = d:get("text")
@@ -689,19 +708,23 @@ function MODEL:createParagraph(pos, width, pinned)
   local d = ipeui.Dialog(self.ui, "Create text object")
   d:add("label", "label", { label="Enter latex source" }, 1, 1, 1, 6)
   d:add("text", "text", { syntax="latex" }, 2, 1, 1, 6)
-  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 5)
-  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 6)
+  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 6)
+  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 5)
   addEditorField(d, "text", 3, 4)
   d:add("style", "combo", styles, 3, 1)
   d:add("size", "combo", sizes, 3, 2)
   d:setStretch("row", 2, 1)
   d:setStretch("column", 3, 1)
+  d:set("ignore-cancel", true)
   local style = indexOf(self.attributes.textstyle, styles)
   if not style then style = indexOf("normal", styles) end
   local size = indexOf(self.attributes.textsize, sizes)
   if not size then size = indexOf("normal", sizes) end
   d:set("style", style)
   d:set("size", size)
+  if prefs.auto_external_editor then
+    externalEditor(d, "text")
+  end
   local r = d:execute(prefs.editor_size)
   if r then
     local t = d:get("text")
@@ -903,44 +926,19 @@ end
 
 ----------------------------------------------------------------------
 
-function MODEL:action_edit_text(prim, obj)
-  local mp = obj:get("minipage")
-  local styles = self.doc:sheets():allNames("textstyle")
-  local sizes = self.doc:sheets():allNames("textsize")
-  local d = ipeui.Dialog(self.ui, "Edit text object")
-  d:add("label", "label", { label="Edit latex source" }, 1, 1, 1, 6)
-  d:add("text", "text", { syntax="latex" }, 2, 1, 1, 6)
-  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 6)
-  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 5)
-  addEditorField(d, "text", 3, 4)
-  d:setStretch("row", 2, 1)
-  d:setStretch("column", 3, 1)
-  d:set("text", obj:text())
-  local style = nil
-  local size = nil
-  if mp then
-    style = indexOf(obj:get("textstyle"), styles)
-    if style then
-      d:add("style", "combo", styles, 3, 1)
-      d:set("style", style)
-    end
-    size = indexOf(obj:get("textsize"), sizes)
-    if size then
-      d:add("size", "combo", sizes, 3, 2)
-      d:set("size", size)
-    end
-  end
-  local r = d:execute(prefs.editor_size)
-  if not r or string.match(d:get("text"), "^%s*$") then return end
-  local final = obj:clone()
+function apply_text_edit(d, data, run_latex)
+  -- refuse to do anything with empty text
+  if string.match(d:get("text"), "^%s*$") then return end
+  local model = data.model
+  local final = data.obj:clone()
   final:setText(d:get("text"))
-  if style then final:set("textstyle", styles[d:get("style")]) end
-  if size then final:set("textsize", sizes[d:get("size")]) end
+  if data.style then final:set("textstyle", data.styles[d:get("style")]) end
+  if data.size then final:set("textsize", data.sizes[d:get("size")]) end
   local t = { label="edit text",
-	      pno=self.pno,
-	      vno=self.vno,
-	      original=obj:clone(),
-	      primary=prim,
+	      pno=model.pno,
+	      vno=model.vno,
+	      original=data.obj:clone(),
+	      primary=data.prim,
 	      final=final,
 	    }
   t.undo = function (t, doc)
@@ -949,7 +947,54 @@ function MODEL:action_edit_text(prim, obj)
   t.redo = function (t, doc)
 	     doc[t.pno]:replace(t.primary, t.final)
 	   end
-  self:register(t)
+  model:register(t)
+  -- need to update data.obj for the next run!
+  data.obj = final
+  if run_latex then model:runLatex() end
+end
+
+function MODEL:action_edit_text(prim, obj)
+  local mp = obj:get("minipage")
+  local d = ipeui.Dialog(self.ui, "Edit text object")
+  local data = { model=self,
+		 styles = self.doc:sheets():allNames("textstyle"),
+		 sizes = self.doc:sheets():allNames("textsize"),
+		 prim=prim,
+		 obj=obj,
+	       }
+  d:add("label", "label", { label="Edit latex source" }, 1, 1, 1, 7)
+  d:add("text", "text", { syntax="latex" }, 2, 1, 1, 7)
+  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 7)
+  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 6)
+  d:add("apply", "button",
+	{ label="&Apply",
+	  action=function (d) apply_text_edit(d, data, true) end },
+	3, 5)
+  addEditorField(d, "text", 3, 4)
+  d:setStretch("row", 2, 1)
+  d:setStretch("column", 3, 1)
+  d:set("text", obj:text())
+  d:set("ignore-cancel", true)
+  local style = nil
+  local size = nil
+  if mp then
+    data.style = indexOf(obj:get("textstyle"), data.styles)
+    if data.style then
+      d:add("style", "combo", data.styles, 3, 1)
+      d:set("style", data.style)
+    end
+    data.size = indexOf(obj:get("textsize"), data.sizes)
+    if data.size then
+      d:add("size", "combo", data.sizes, 3, 2)
+      d:set("size", data.size)
+    end
+  end
+  if prefs.auto_external_editor then
+    externalEditor(d, "text")
+  end
+  local r = d:execute(prefs.editor_size)
+  if not r or string.match(d:get("text"), "^%s*$") then return end
+  apply_text_edit(d, data, false)
 end
 
 function MODEL:action_edit()
@@ -980,15 +1025,26 @@ function PASTETOOL:new(model, elements, pos)
   tool.start = model.ui:pos()
   if pos then tool.start = pos end
   local obj = ipe.Group(elements)
+  tool.pinned = obj:get("pinned")
   model.ui:pasteTool(obj, tool)
   tool.setColor(1.0, 0, 0)
-  tool.translation = model.ui:pos() - tool.start
+  tool:computeTranslation()
   tool.setMatrix(ipe.Translation(tool.translation))
   return tool
 end
 
-function PASTETOOL:mouseButton(button, modifiers, press)
+function PASTETOOL:computeTranslation()
   self.translation = self.model.ui:pos() - self.start
+  if self.pinned == "horizontal" or self.pinned == "fixed" then
+    self.translation = V(0, self.translation.y)
+  end
+  if self.pinned == "vertical" or self.pinned == "fixed" then
+    self.translation = V(self.translation.x, 0)
+  end
+end
+
+function PASTETOOL:mouseButton(button, modifiers, press)
+  self:computeTranslation()
   self.model.ui:finishTool()
   local t = { label="paste objects at cursor",
 	      pno = self.model.pno,
@@ -1014,7 +1070,7 @@ function PASTETOOL:mouseButton(button, modifiers, press)
 end
 
 function PASTETOOL:mouseMove(button, modifiers)
-  self.translation = self.model.ui:pos() - self.start
+  self:computeTranslation()
   self.setMatrix(ipe.Translation(self.translation))
   self.model.ui:update(false) -- update tool
 end
