@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,11 +41,12 @@ extern "C" {
 
 #include <cstdio>
 #include <cstdlib>
-#include <clocale>
+#include <locale.h>
 
 #include <QApplication>
 #include <QLocale>
 #include <QDir>
+#include <QDesktopWidget>
 
 extern int luaopen_ipeui(lua_State *L);
 extern int luaopen_appui(lua_State *L);
@@ -92,9 +93,6 @@ static void setup_config(lua_State *L, const char *var,
 int main(int argc, char *argv[])
 {
   Platform::initLib(IPELIB_VERSION);
-  const char *qts = getenv("IPEQTSTYLE");
-  if (qts)
-    QApplication::setStyle(qts);
   QApplication a(argc, argv);
   setlocale(LC_NUMERIC, "C");
   a.setQuitOnLastWindowClosed(true);
@@ -158,6 +156,14 @@ int main(int argc, char *argv[])
 		  IPELIB_VERSION % 100);
   lua_setfield(L, -2, "version");
 
+  QRect r = a.desktop()->screenGeometry();
+  lua_createtable(L, 0, 2);
+  lua_pushinteger(L, r.width());
+  lua_rawseti(L, -2, 1);
+  lua_pushinteger(L, r.height());
+  lua_rawseti(L, -2, 2);
+  lua_setfield(L, -2, "screen_geometry");
+
   lua_setglobal(L, "config");
 
   // run Ipe
