@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -35,16 +35,26 @@ extern "C" {
 #include "lfs.h"
 }
 
+#include "ipeselector.h"
+
 #include "appui.h"
 #include "tools.h"
+#include "pagesorter.h"
 
 #include "ipelua.h"
 
+#include "pagesorter.h"
+
 #include <cstdio>
 #include <cstdlib>
 
 #include <QApplication>
 #include <QStatusBar>
+#include <QPainter>
+
+#include <QDialog>
+#include <QVBoxLayout>
+#include <QDialogButtonBox>
 
 using namespace ipe;
 using namespace ipeqt;
@@ -86,9 +96,10 @@ static int appui_setPage(lua_State *L)
 {
   AppUi **ui = check_appui(L, 1);
   Page *page = check_page(L, 2)->page;
-  int view = check_viewno(L, 3, page);
-  Cascade *sheets = check_cascade(L, 4)->cascade;
-  (*ui)->canvas()->setPage(page, view, sheets);
+  int pno = luaL_checkinteger(L, 3) - 1;
+  int view = check_viewno(L, 4, page);
+  Cascade *sheets = check_cascade(L, 5)->cascade;
+  (*ui)->canvas()->setPage(page, pno, view, sheets);
   return 0;
 }
 
@@ -159,13 +170,6 @@ static int appui_simpleSnapPos(lua_State *L)
   return 1;
 }
 
-static int appui_setPretty(lua_State *L)
-{
-  Canvas *canvas = check_canvas(L, 1);
-  canvas->setPretty(lua_toboolean(L, 2));
-  return 0;
-}
-
 static int appui_setFifiVisible(lua_State *L)
 {
   Canvas *canvas = check_canvas(L, 1);
@@ -259,6 +263,56 @@ static int appui_canvasSize(lua_State *L)
   return 1;
 }
 
+static void check_rgb(lua_State *L, int i, int &r, int &g, int &b)
+{
+  r = int(1000 * luaL_checknumber(L, i) + 0.5);
+  g = int(1000 * luaL_checknumber(L, i+1) + 0.5);
+  b = int(1000 * luaL_checknumber(L, i+2) + 0.5);
+  luaL_argcheck(L, (0 <= r && r <= 1000 &&
+		    0 <= g && g <= 1000 &&
+		    0 <= b && b <= 1000), 2,
+		"color components must be between 0.0 and 1.0");
+}
+
+static int appui_setCursor(lua_State *L)
+{
+  Canvas *canvas = check_canvas(L, 1);
+  if (lua_isnumber(L, 2)) {
+    double s = lua_tonumber(L, 2);
+    int r, g, b;
+    check_rgb(L, 3, r, g, b);
+    QPixmap p(32, 32);
+    p.fill(QColor(255, 255, 255, 0));
+    QPainter painter(&p);
+    s = 0.5 * s * canvas->zoom();
+    if (s < 1.0)
+      s = 1.0;
+    if (s > 10.0)
+      s = 10.0;
+    r = 255 * r / 1000;
+    g = 255 * g / 1000;
+    b = 255 * b / 1000;
+    painter.setBrush(QColor(r, g, b));
+    painter.setPen(Qt::NoPen);
+    painter.drawEllipse(QRectF(16.0 - s, 16.0 - s, 2 * s, 2 * s));
+    painter.end();
+    QCursor cursor(p);
+    canvas->setCursor(cursor);
+  } else
+    canvas->unsetCursor();
+  return 0;
+}
+
+static int appui_setNumbering(lua_State *L)
+{
+  Canvas *canvas = check_canvas(L, 1);
+  bool t = lua_toboolean(L, 2);
+  Canvas::Style s = canvas->canvasStyle();
+  s.numberPages = t;
+  canvas->setCanvasStyle(s);
+  return 0;
+}
+
 // --------------------------------------------------------------------
 
 static int appui_pantool(lua_State *L)
@@ -311,13 +365,8 @@ static int appui_transformtool(lua_State *L)
 static int luatool_setcolor(lua_State *L)
 {
   LuaTool *tool = (LuaTool *) lua_touserdata(L, lua_upvalueindex(1));
-  int r = int(1000 * luaL_checknumber(L, 1) + 0.5);
-  int g = int(1000 * luaL_checknumber(L, 2) + 0.5);
-  int b = int(1000 * luaL_checknumber(L, 3) + 0.5);
-  luaL_argcheck(L, (0 <= r && r <= 1000 &&
-		    0 <= g && g <= 1000 &&
-		    0 <= b && b <= 1000), 2,
-		"color components must be between 0.0 and 1.0");
+  int r, g, b;
+  check_rgb(L, 1, r, g, b);
   tool->setColor(Color(r, g, b));
   return 0;
 }
@@ -478,9 +527,11 @@ static int appui_setNumbers(lua_State *L)
   String pno;
   if (!lua_isnil(L, 2))
     vno = luaL_checkstring(L, 2);
-  if (!lua_isnil(L, 3))
-    pno = luaL_checkstring(L, 3);
-  (*ui)->setNumbers(vno, pno);
+  bool vm = lua_toboolean(L, 3);
+  if (!lua_isnil(L, 4))
+    pno = luaL_checkstring(L, 4);
+  bool pm = lua_toboolean(L, 5);
+  (*ui)->setNumbers(vno, vm, pno, pm);
   return 0;
 }
 
@@ -498,14 +549,138 @@ static int appui_setWindowTitle(lua_State *L)
   return 0;
 }
 
+static int appui_setNotes(lua_State *L)
+{
+  AppUi **ui = check_appui(L, 1);
+  const char *s = luaL_checkstring(L, 2);
+  (*ui)->setNotes(s);
+  return 0;
+}
+
+static int appui_showTool(lua_State *L)
+{
+  AppUi **ui = check_appui(L, 1);
+  return (*ui)->showTool(L);
+}
+
 static int appui_explain(lua_State *L)
 {
   AppUi **ui = check_appui(L, 1);
   const char *s = luaL_checkstring(L, 2);
-  (*ui)->statusBar()->showMessage(QString::fromUtf8(s), 4000);
+  int t = 4000;
+  if (lua_isnumber(L, 3))
+    t = lua_tointeger(L, 3);
+  (*ui)->statusBar()->showMessage(QString::fromUtf8(s), t);
   return 0;
 }
 
+static int appui_checkbox(lua_State *L)
+{
+  AppUi **ui = check_appui(L, 1);
+  lua_pushboolean(L, (*ui)->checkbox(L));
+  return 1;
+}
+
+// --------------------------------------------------------------------
+
+static void get_page_sorter_size(lua_State *L, int &width, int &height,
+				 int &thumbWidth)
+{
+  thumbWidth = 300;
+  width = 600;
+  height = 480;
+
+  lua_getglobal(L, "prefs");
+  lua_getfield(L, -1, "page_sorter_size");
+  if (lua_istable(L, -1)) {
+    lua_rawgeti(L, -1, 1);
+    if (lua_isnumber(L, -1))
+      width = lua_tointeger(L, -1);
+    lua_rawgeti(L, -2, 2);
+    if (lua_isnumber(L, -1))
+      height = lua_tointeger(L, -1);
+    lua_pop(L, 2);
+  }
+  lua_pop(L, 1); // page_sorter_size
+
+  lua_getfield(L, -1, "thumbnail_width");
+  if (lua_isnumber(L, -1))
+    thumbWidth = lua_tointeger(L, -1);
+  lua_pop(L, 1); // thumbnail_width, prefs
+}
+
+static int appui_selectPage(lua_State *L)
+{
+  AppUi **ui = check_appui(L, 1);
+  (void) ui;  // not actually used now
+  Document **doc = check_document(L, 2);
+  int page = -1;
+  if (lua_isnumber(L, 3)) {
+    page = lua_tointeger(L, 3);
+    luaL_argcheck(L, 1 <= page && page <= (*doc)->countPages(),
+		  3, "invalid page number");
+  }
+
+  int startIndex = 1;
+  if (lua_isnumber(L, 4)) {
+    startIndex = lua_tointeger(L, 4);
+    int maxIndex = (page < 0) ? (*doc)->countPages() :
+      (*doc)->page(page-1)->countViews();
+    luaL_argcheck(L, 1 <= startIndex && startIndex <= maxIndex,
+		  4, "invalid start index");
+  }
+
+  int width, height, thumbWidth;
+  get_page_sorter_size(L, width, height, thumbWidth);
+
+  int sel = PageSelector::selectPageOrView(*doc, page - 1, startIndex - 1,
+					   thumbWidth, width, height);
+  if (sel >= 0) {
+    lua_pushinteger(L, sel + 1);
+    return 1;
+  } else
+    return 0;
+}
+
+static int appui_pageSorter(lua_State *L)
+{
+  AppUi **ui = check_appui(L, 1);
+  (void) ui;  // not actually used now
+  Document **doc = check_document(L, 2);
+
+  int width, height, thumbWidth;
+  get_page_sorter_size(L, width, height, thumbWidth);
+
+  QDialog *d = new QDialog();
+  d->setWindowTitle("Ipe Page Sorter");
+
+  QLayout *lo = new QVBoxLayout;
+  PageSorter *p = new PageSorter(*doc, thumbWidth);
+  QDialogButtonBox *buttonBox =
+    new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
+  lo->addWidget(p);
+  lo->addWidget(buttonBox);
+  d->setLayout(lo);
+
+  d->connect(buttonBox, SIGNAL(accepted()), SLOT(accept()));
+  d->connect(buttonBox, SIGNAL(rejected()), SLOT(reject()));
+
+  d->resize(width, height);
+
+  if (d->exec() == QDialog::Rejected) {
+    delete d;
+    return 0;
+  }
+
+  lua_createtable(L, p->count(), 0);
+  for (int i = 1; i <= p->count(); ++i) {
+    lua_pushinteger(L, p->pageAt(i-1) + 1);
+    lua_rawseti(L, -2, i);
+  }
+  delete d;
+  return 1;
+}
+
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg appui_methods[] = {
@@ -522,13 +697,14 @@ static const struct luaL_Reg appui_methods[] = {
   { "globalPos", appui_globalPos},
   { "unsnappedPos", appui_unsnappedPos},
   { "simpleSnapPos", appui_simpleSnapPos },
-  { "setPretty", appui_setPretty},
   { "setFifiVisible", appui_setFifiVisible},
   { "setSnap", appui_setSnap},
   { "setAutoOrigin", appui_setAutoOrigin },
   { "update", appui_update},
   { "finishTool", appui_finishTool},
   { "canvasSize", appui_canvasSize },
+  { "setCursor", appui_setCursor },
+  { "setNumbering", appui_setNumbering },
   // --------------------------------------------------------------------
   { "panTool", appui_pantool },
   { "selectTool", appui_selecttool },
@@ -540,6 +716,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "setActionState", appui_setActionState },
   { "actionState", appui_actionState },
   { "explain", appui_explain },
+  { "checkbox", appui_checkbox },
   { "setWindowTitle", appui_setWindowTitle },
   { "setupSymbolicNames", appui_setupSymbolicNames },
   { "setAttributes", appui_setAttributes },
@@ -547,6 +724,10 @@ static const struct luaL_Reg appui_methods[] = {
   { "setLayers", appui_setLayers },
   { "setNumbers", appui_setNumbers },
   { "setBookmarks", appui_setBookmarks },
+  { "setNotes", appui_setNotes },
+  { "showTool", appui_showTool },
+  { "selectPage", appui_selectPage },
+  { "pageSorter", appui_pageSorter },
   { 0, 0},
 };
 
@@ -571,27 +752,45 @@ static int appui_constructor(lua_State *L)
   style.classicGrid = false;
   style.thinLine = 0.2;
   style.thickLine = 0.9;
+  style.thinStep = 1;
+  style.thickStep = 4;
+  style.paperClip = false;
+  style.numberPages = false;
 
   lua_getglobal(L, "prefs");
 
-  lua_getfield(L, -1, "paper_color");
-  if (!lua_isnil(L, -1))
-    style.paperColor = check_color(L, lua_gettop(L));
-  lua_pop(L, 1); // paper_color
+  lua_getfield(L, -1, "canvas_style");
+  if (!lua_isnil(L, -1)) {
+    lua_getfield(L, -1, "paper_color");
+    if (!lua_isnil(L, -1))
+      style.paperColor = check_color(L, lua_gettop(L));
+    lua_pop(L, 1); // paper_color
 
-  lua_getfield(L, -1, "classic_grid");
-  style.classicGrid = lua_toboolean(L, -1);
-  lua_pop(L, 1); // classic_grid
+    lua_getfield(L, -1, "classic_grid");
+    style.classicGrid = lua_toboolean(L, -1);
+    lua_pop(L, 1); // classic_grid
 
-  lua_getfield(L, -1, "thin_grid_line");
-  if (lua_isnumber(L, -1))
-    style.thinLine = lua_tonumber(L, -1);
-  lua_pop(L, 1); // thin_grid_line
+    lua_getfield(L, -1, "thin_grid_line");
+    if (lua_isnumber(L, -1))
+      style.thinLine = lua_tonumber(L, -1);
+    lua_pop(L, 1); // thin_grid_line
 
-  lua_getfield(L, -1, "thick_grid_line");
-  if (lua_isnumber(L, -1))
-    style.thickLine = lua_tonumber(L, -1);
-  lua_pop(L, 1); // thick_grid_line
+    lua_getfield(L, -1, "thick_grid_line");
+    if (lua_isnumber(L, -1))
+      style.thickLine = lua_tonumber(L, -1);
+    lua_pop(L, 1); // thick_grid_line
+
+    lua_getfield(L, -1, "thin_step");
+    if (lua_isnumber(L, -1))
+      style.thinStep = lua_tointeger(L, -1);
+    lua_pop(L, 1); // thin_step
+
+    lua_getfield(L, -1, "thick_step");
+    if (lua_isnumber(L, -1))
+      style.thickStep = lua_tointeger(L, -1);
+    lua_pop(L, 1); // thick_step
+  }
+  lua_pop(L, 1); // canvas_style
 
   int width = -1, height = -1;
   lua_getfield(L, -1, "window_size");
