@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -292,7 +292,8 @@ bool ImageIpelet::insertBitmap(QString name)
     bitmap.setColorKey(colorKey);
 
   Image *obj = new Image(computeRect(), bitmap);
-  iData->iPage->append(ESecondarySelected, iData->iLayer, obj);
+  iData->iPage->deselectAll();
+  iData->iPage->append(EPrimarySelected, iData->iLayer, obj);
   return true;
 }
 
@@ -426,7 +427,8 @@ bool ImageIpelet::insertJpeg(QString name)
 		iBitsPerComponent, Buffer(a.data(), a.size()),
 		Bitmap::EDCTDecode);
   Image *obj = new Image(computeRect(), bitmap);
-  iData->iPage->append(ESecondarySelected, iData->iLayer, obj);
+  iData->iPage->deselectAll();
+  iData->iPage->append(EPrimarySelected, iData->iLayer, obj);
   return true;
 }
 
