@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -232,6 +232,13 @@ int Repository::toIndex(String str)
   return iStrings.size() - 1;
 }
 
+//! Destroy repository object.
+void Repository::cleanup()
+{
+  delete singleton;
+  singleton = 0;
+}
+
 // --------------------------------------------------------------------
 
 /*! \class ipe::Attribute
