@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -40,6 +40,12 @@
 #include "ipepswriter.h"
 #include "ipelatex.h"
 
+#include <errno.h>
+
+#ifdef IPE_USE_ICONV
+#include <iconv.h>
+#endif
+
 using namespace ipe;
 
 // --------------------------------------------------------------------
@@ -438,7 +444,7 @@ bool Document::save(TellStream &stream, TFormat format, uint flags) const
     compresslevel = 0;
 
   if (format == EPdf) {
-    PdfWriter writer(stream, this, iFontPool, (flags & ELastView),
+    PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
 		     0, -1, compresslevel);
     writer.createPages();
     writer.createBookmarks();
@@ -505,7 +511,7 @@ bool Document::exportView(const char *fname, TFormat format, uint flags,
   FileStream stream(fd);
 
   if (format == EPdf) {
-    PdfWriter writer(stream, this, iFontPool, (flags & ELastView),
+    PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
 		     pno, pno, compresslevel);
     writer.createPageView(pno, vno);
     writer.createTrailer();
@@ -534,7 +540,7 @@ bool Document::exportPages(const char *fname, uint flags,
   if (!fd)
     return false;
   FileStream stream(fd);
-  PdfWriter writer(stream, this, iFontPool, (flags & ELastView),
+  PdfWriter writer(stream, this, iFontPool, (flags & EMarkedView),
 		   fromPage, toPage, compresslevel);
   writer.createPages();
   writer.createTrailer();
@@ -562,7 +568,7 @@ void Document::findBitmaps(BitmapFinder &bm) const
 //! Save in XML format into an Stream.
 void Document::saveAsXml(Stream &stream, bool usePdfBitmaps) const
 {
-  stream << "<ipe version=\"" << ipe::IPELIB_VERSION << "\"";
+  stream << "<ipe version=\"" << ipe::FILE_FORMAT << "\"";
   if (!iProperties.iCreator.empty())
     stream << " creator=\"" << iProperties.iCreator << "\"";
   stream << ">\n";
@@ -793,10 +799,14 @@ int Document::runLatex(String &texLog)
   texLog = "";
   Latex converter(cascade());
 
-  const Symbol *background =
-    cascade()->findSymbol(Attribute::BACKGROUND());
-  if (background)
-    converter.scanObject(background->iObject);
+  AttributeSeq seq;
+  cascade()->allNames(ESymbol, seq);
+
+  for (AttributeSeq::iterator it = seq.begin(); it != seq.end(); ++it) {
+    const Symbol *sym = cascade()->findSymbol(*it);
+    if (sym)
+      converter.scanObject(sym->iObject);
+  }
 
   int count = 0;
   for (int i = 0; i < countPages(); ++i)
@@ -809,12 +819,62 @@ int Document::runLatex(String &texLog)
   if (latexDir.empty())
     return ErrNoDir;
 
-  String texFile = latexDir + "text.tex";
-  String pdfFile = latexDir + "text.pdf";
-  String logFile = latexDir + "text.log";
+  String texFile = latexDir + "ipetemp.tex";
+  String pdfFile = latexDir + "ipetemp.pdf";
+  String logFile = latexDir + "ipetemp.log";
 
   std::remove(logFile.z());
 
+  String encoding = cascade()->findEncoding();
+
+#ifdef IPE_USE_ICONV
+  String utf8;
+  StringStream stream(utf8);
+  int err = converter.createLatexSource(stream, properties().iPreamble);
+  if (err < 0)
+    return ErrWritingSource;
+
+  if (encoding.empty()) {
+    std::FILE *file = std::fopen(texFile.z(), "wb");
+    if (!file)
+      return ErrWritingSource;
+    FileStream fstream(file);
+    fstream.putRaw(utf8.data(), utf8.size());
+    std::fclose(file);
+  } else {
+    iconv_t conv = iconv_open(encoding.z(), "UTF-8");
+    if (conv == iconv_t(-1))
+      return ErrWritingSource;
+
+    std::FILE *file = std::fopen(texFile.z(), "wb");
+    if (!file)
+      return ErrWritingSource;
+
+    char *inbuf = (char *) utf8.data();
+    size_t inbytesleft = utf8.size();
+
+    FileStream fstream(file);
+    while (inbytesleft > 0) {
+      char outbuf[0x100];
+      char *outp = outbuf;
+      size_t outbytesleft = 0x100;
+      size_t result = iconv(conv, &inbuf, &inbytesleft, &outp, &outbytesleft);
+      // E2BIG means output buffer exhausted, we continue in the next round
+      if (result == size_t(-1) && errno != E2BIG) {
+	std::fclose(file);
+	iconv_close(conv);
+	return ErrWritingSource;
+      }
+      if (outp > outbuf)
+	fstream.putRaw(outbuf, outp - outbuf);
+    }
+    iconv_close(conv);
+    std::fclose(file);
+  }
+#else
+  if (!encoding.empty())
+    return ErrNoIconv;
+
   std::FILE *file = std::fopen(texFile.z(), "wb");
   if (!file)
     return ErrWritingSource;
@@ -824,6 +884,7 @@ int Document::runLatex(String &texLog)
 
   if (err < 0)
     return ErrWritingSource;
+#endif
 
   int result = Platform::runPdfLatex(latexDir);
 
@@ -886,6 +947,11 @@ int Document::runLatex()
   case ErrLatexOutput:
     fprintf(stderr, "There was an error reading the Pdflatex output.\n");
     return 1;
+  case ErrNoIconv:
+    fprintf(stderr,
+	    "This document needs charset conversion to run Pdflatex,\n"
+	    "but Ipe is compiled without this feature.\n");
+    return 1;
   case ErrNone:
   default:
     fprintf(stderr, "Pdflatex was run sucessfully.\n");
