@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -123,6 +123,7 @@ int ImlParser::parseDocument(Document &doc)
       return ESyntaxError;
     if (!parsePCDATA("preamble", properties.iPreamble))
       return ESyntaxError;
+
     tag = parseToTag();
   }
 
@@ -198,12 +199,28 @@ bool ImlParser::parsePage(Page &page)
     page.setSection(0, str.empty(), str);
   else
     page.setSection(0, false, String());
+
   if (att.has("subsection", str))
     page.setSection(1, str.empty(), str);
   else
     page.setSection(1, false, String());
 
+  if (att["marked"] == "no")
+    page.setMarked(false);
+
   String tag = parseToTag();
+
+  if (tag == "notes") {
+    XmlAttributes att;
+    if (!parseAttributes(att))
+      return ESyntaxError;
+    if (!parsePCDATA("notes", str))
+      return ESyntaxError;
+    page.setNotes(str);
+
+    tag = parseToTag();
+  }
+
   while (tag == "layer") {
     XmlAttributes att;
     if (!parseAttributes(att))
@@ -228,15 +245,23 @@ bool ImlParser::parsePage(Page &page)
       page.setEffect(page.countViews() - 1, Attribute(true, str));
 
     Lex st(att["layers"]);
+    st.skipWhitespace();
     String last;
-    do {
+    while (!st.eos()) {
       last = st.nextToken();
-      st.skipWhitespace();
       page.setVisible(page.countViews() - 1, last, true);
-    } while (!st.eos());
+      st.skipWhitespace();
+    }
 
-    if (!att.has("active", str))
+    if (!att.has("active", str)) {
+      // if no layer visible must have active attribute
+      if (last.empty())
+	return false;
       page.setActive(page.countViews() - 1, last);
+    }
+
+    if (att["marked"] == "yes")
+      page.setMarkedView(page.countViews() - 1, true);
 
     tag = parseToTag();
   }
@@ -474,6 +499,7 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
     } else if (tag == "preamble") {
       if (!parseAttributes(att))
 	return false;
+      sheet.setEncoding(att["encoding"]);
       String pcdata;
       if (!att.slash() && !parsePCDATA(tag, pcdata))
 	return false;
@@ -637,7 +663,7 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
       String name = att["name"];
       Attribute value = Attribute::makeScalar(att["value"],
 					      Attribute::NORMAL());
-      if (name.isEmpty() || value.isSymbolic())
+      if (name.empty() || value.isSymbolic())
 	return false;
       sheet.add(kind, Attribute(true, name), value);
     }
