@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -152,7 +152,9 @@ int Latex::createLatexSource(Stream &stream, String preamble)
   bool ancient = (getenv("IPEANCIENTPDFTEX") != 0);
   int count = 0;
   stream << "\\pdfcompresslevel0\n"
-	 << "\\nonstopmode\n";
+	 << "\\nonstopmode\n"
+	 << "\\ifx\\pdfobjcompresslevel\\relax\\else"
+	 << "\\pdfobjcompresslevel0\\fi\n";
   if (!ancient) {
     stream << "\\ifnum\\the\\pdftexversion<140"
 	   << "\\errmessage{Pdftex is too old. "
