@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -77,6 +77,7 @@ using namespace ipe;
 Page::Page() : iTitle()
 {
   iUseTitle[0] = iUseTitle[1] = false;
+  iMarked = true;
 }
 
 //! Create a new empty page with standard settings.
@@ -116,7 +117,15 @@ void Page::saveAsXml(Stream &stream) const
     stream.putXmlString(iSection[1]);
     stream << "\"";
   }
+  if (!iMarked)
+    stream << " marked=\"no\"";
+
   stream << ">\n";
+  if (!iNotes.empty()) {
+    stream << "<notes>";
+    stream.putXmlString(iNotes);
+    stream << "</notes>\n";
+  }
   for (int i = 0; i < countLayers(); ++i) {
     stream << "<layer name=\"" << iLayers[i].iName << "\"";
     if (iLayers[i].iFlags & ELocked)
@@ -137,6 +146,8 @@ void Page::saveAsXml(Stream &stream) const
       stream << " active=\"" << active(i) << "\"";
     if (!effect(i).isNormal())
       stream << " effect=\"" << effect(i).string() << "\"";
+    if (markedView(i))
+      stream << " marked=\"yes\"";
     stream << "/>\n";
   }
   int currentLayer = -1;
@@ -317,6 +328,7 @@ void Page::insertView(int i, String active)
 {
   iViews.insert(iViews.begin() + i, SView());
   iViews[i].iActive = active;
+  iViews[i].iMarked = false;
   for (int l = 0; l < countLayers(); ++l)
     iLayers[l].iVisible.insert(iLayers[l].iVisible.begin() + i, false);
 }
@@ -338,6 +350,21 @@ void Page::clearViews()
     it->iVisible.clear();
 }
 
+void Page::setMarkedView(int index, bool marked)
+{
+  iViews[index].iMarked = marked;
+}
+
+int Page::countMarkedViews() const
+{
+  int count = 0;
+  for (int i = 0; i < countViews(); ++i) {
+    if (markedView(i))
+      ++count;
+  }
+  return (count == 0) ? 1 : count;
+}
+
 // --------------------------------------------------------------------
 
 Page::SObject::SObject()
@@ -550,7 +577,7 @@ void Page::setSection(int level, bool useTitle, String name)
   iSection[level] = useTitle ? String() : name;
 }
 
-//! set the title of this page.
+//! Set the title of this page.
 /*! An empty title is not displayed. */
 void Page::setTitle(String title)
 {
@@ -564,6 +591,18 @@ String Page::title() const
   return iTitle;
 }
 
+//! Set the notes of this page.
+void Page::setNotes(String notes)
+{
+  iNotes = notes;
+}
+
+//! Set if page is marked for printing.
+void Page::setMarked(bool marked)
+{
+  iMarked = marked;
+}
+
 //! Return Text object representing the title text.
 /*! Return 0 if no title is set.
   Ownership of object remains with Page.
