@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -79,11 +79,11 @@ void PsPainter::strokePath()
   }
   if (s.iLineCap != sa.iLineCap) {
     sa.iLineCap = s.iLineCap;
-    iStream << s.iLineCap << " J\n";
+    iStream << s.iLineCap - 1 << " J\n";
   }
   if (s.iLineJoin != sa.iLineJoin) {
     sa.iLineJoin = s.iLineJoin;
-    iStream << s.iLineJoin << " j\n";
+    iStream << s.iLineJoin - 1 << " j\n";
   }
   if (s.iStroke != sa.iStroke) {
     sa.iStroke = s.iStroke;
