@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -440,7 +440,13 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
       stream << " value=\"" << it->second.string() << "\"/>\n";
   }
   if (!iPreamble.empty()) {
-    stream << "<preamble>";
+    stream << "<preamble";
+    if (!iEncoding.empty()) {
+      stream << " encoding=\"";
+      stream.putXmlString(iEncoding);
+      stream << "\"";
+    }
+    stream << ">";
     stream.putXmlString(iPreamble);
     stream << "</preamble>\n";
   }
@@ -454,6 +460,12 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
       stream << "\" crop=\"no";
     stream << "\"/>\n";
   }
+  if (iPageNumberStyle.iDefined) {
+    stream << "<pagenumberstyle pos=\"" << iPageNumberStyle.iPos
+	   << "\" size=\"" << iPageNumberStyle.iFontSize
+	   << "\" color=\"" << iPageNumberStyle.iColor
+	   << "\"/>\n";
+  }
   if (iTitleStyle.iDefined) {
     stream << "<titlestyle pos=\"" << iTitleStyle.iPos
 	   << "\" size=\"" << iTitleStyle.iSize.string()
@@ -702,6 +714,16 @@ String Cascade::findPreamble() const
   return s;
 }
 
+//! Return LaTeX encoding (of the whole cascade).
+String Cascade::findEncoding() const
+{
+  for (int i = 0; i < count(); ++i) {
+    if (!iSheets[i]->encoding().isEmpty())
+      return iSheets[i]->encoding();
+  }
+  return String();
+}
+
 TLineCap Cascade::lineCap() const
 {
   for (int i = 0; i < count(); ++i) {
