@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -191,7 +191,7 @@ static int document_pages(lua_State *L)
   return 3;
 }
 
-// "export", "nozip", "lastview", "nocolor"
+// "export", "nozip", "markedview", "nocolor"
 static uint check_flags(lua_State *L, int index)
 {
   if (lua_isnoneornil(L, index))
@@ -206,9 +206,9 @@ static uint check_flags(lua_State *L, int index)
   if (lua_toboolean(L, -1))
     flags |= Document::ENoZip;
   lua_pop(L, 1);
-  lua_getfield(L, index, "lastview");
+  lua_getfield(L, index, "markedview");
   if (lua_toboolean(L, -1))
-    flags |= Document::ELastView;
+    flags |= Document::EMarkedView;
   lua_pop(L, 1);
   lua_getfield(L, index, "nocolor");
   if (lua_toboolean(L, -1))
@@ -360,6 +360,11 @@ static int document_runLatex(lua_State *L)
       lua_pushliteral(L, "There was an error reading the Pdflatex output");
       lua_pushliteral(L, "latexoutput");
       break;
+    case Document::ErrNoIconv:
+      lua_pushliteral(L, "This document needs charset conversion to run "
+		      "Pdflatex, but Ipe is compiled without this feature");
+      lua_pushliteral(L, "noiconv");
+      break;
     }
   }
   push_string(L, log);
@@ -395,16 +400,14 @@ static int document_insert(lua_State *L)
   int no = check_pageno(L, 2, *d, 1);
   SPage *p = check_page(L, 3);
   (*d)->insert(no, new Page(*p->page));
-  p->owned = false;
   return 0;
 }
 
 static int document_append(lua_State *L)
 {
   Document **d = check_document(L, 1);
-  SPage *p = check_page(L, 3);
+  SPage *p = check_page(L, 2);
   (*d)->push_back(new Page(*p->page));
-  p->owned = false;
   return 0;
 }
 
