@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -159,11 +159,17 @@ Canvas::Canvas(QWidget* parent, Qt::WFlags f)
   iAutoSnap = false;
   iFifiVisible = false;
 
+  iAdditionalModifiers = 0;
+
   iStyle.paperColor = Color(1000,1000,1000);
   iStyle.pretty = false;
   iStyle.classicGrid = false;
   iStyle.thinLine = 0.2;
   iStyle.thickLine = 0.9;
+  iStyle.thinStep = 1;
+  iStyle.thickStep = 4;
+  iStyle.paperClip = false;
+  iStyle.numberPages = false;
 
   iSnap.iSnap = 0;
   iSnap.iGridVisible = false;
@@ -201,10 +207,13 @@ void Canvas::setFontPool(const FontPool *fontPool)
 // --------------------------------------------------------------------
 
 //! Set the page to be displayed.
-/*! Doesn't take ownership of any argument. */
-void Canvas::setPage(const Page *page, int view, const Cascade *sheet)
+/*! Doesn't take ownership of any argument.
+  The page number \a pno is only needed if page numbering is turned on.
+*/
+void Canvas::setPage(const Page *page, int pno, int view, const Cascade *sheet)
 {
   iPage = page;
+  iPageNumber = pno;
   iView = view;
   iCascade = sheet;
 }
@@ -216,14 +225,6 @@ void Canvas::setCanvasStyle(const Style &style)
   iStyle = style;
 }
 
-//! Determine whether pretty display should be used.
-/*! In pretty display, no dashed lines are drawn around text objects,
-  and if Latex font data is not available, no text is drawn at all. */
-void Canvas::setPretty(bool pretty)
-{
-  iStyle.pretty = pretty;
-}
-
 //! Set current pan position.
 /*! The pan position is the user coordinate that is displayed at
   the very center of the canvas. */
@@ -252,6 +253,15 @@ void Canvas::setDimmed(bool dimmed)
   iDimmed = dimmed;
 }
 
+//! Set additional modifiers.
+/*! These modifier bits are passed to the Tool when a key is pressed
+    or a drawing action is performed in addition to the actual
+    keyboard modifiers. */
+void Canvas::setAdditionalModifiers(int mod)
+{
+  iAdditionalModifiers = mod;
+}
+
 //! Enable automatic angular snapping with this origin.
 void Canvas::setAutoOrigin(const Vector &v)
 {
@@ -327,17 +337,16 @@ void Canvas::drawAxes(cairo_t *cc)
 
 void Canvas::drawGrid(cairo_t *cc)
 {
-  int step = iSnap.iGridSize;
+  int step = iSnap.iGridSize * iStyle.thinStep;
   double pixstep = step * iZoom;
   if (pixstep < 3.0)
     return;
 
-  int vfactor = 1;
-  int vstep = step * vfactor;
-
-  Rect paper = iCascade->findLayout()->paper();
-  Vector ll = paper.bottomLeft();
-  Vector ur = paper.topRight();
+  // Rect paper = iCascade->findLayout()->paper();
+  // Vector ll = paper.bottomLeft();
+  // Vector ur = paper.topRight();
+  Vector ll = Vector::ZERO;
+  Vector ur = iCascade->findLayout()->iFrameSize;
 
   int left = step * int(ll.x / step);
   if (left < ll.x)
@@ -356,9 +365,9 @@ void Canvas::drawGrid(cairo_t *cc)
   if (iStyle.classicGrid) {
     double lw = iStyle.thinLine / iZoom;
     cairo_set_line_width(cc, lw);
-    for (int y = bottom; y < ur.y; y += vstep) {
+    for (int y = bottom; y < ur.y; y += step) {
       if (screenLR.y <= y && y <= screenUL.y) {
-	for (int x = left; x < ur.x; x += vstep) {
+	for (int x = left; x < ur.x; x += step) {
 	  if (screenUL.x <= x && x <= screenLR.x) {
 	    cairo_move_to(cc, x, y - 0.5 * lw);
 	    cairo_line_to(cc, x, y + 0.5 * lw);
@@ -370,10 +379,10 @@ void Canvas::drawGrid(cairo_t *cc)
   } else {
     double thinLine = iStyle.thinLine / iZoom;
     double thickLine = iStyle.thickLine / iZoom;
-    int thickStep = 4 * step;
+    int thickStep = iStyle.thickStep * step;
 
     // draw horizontal lines
-    for (int y = bottom; y < ur.y; y += vstep) {
+    for (int y = bottom; y < ur.y; y += step) {
       if (screenLR.y <= y && y <= screenUL.y) {
 	cairo_set_line_width(cc, (y % thickStep) ? thinLine : thickLine);
 	cairo_move_to(cc, ll.x, y);
@@ -383,7 +392,7 @@ void Canvas::drawGrid(cairo_t *cc)
     }
 
     // draw vertical lines
-    for (int x = left; x < ur.x; x += vstep) {
+    for (int x = left; x < ur.x; x += step) {
       if (screenUL.x <= x && x <= screenLR.x) {
 	cairo_set_line_width(cc, (x % thickStep) ? thinLine : thickLine);
 	cairo_move_to(cc, x, ll.y);
@@ -429,6 +438,13 @@ void Canvas::drawObjects(cairo_t *cc)
   if (!iPage)
     return;
 
+  if (iStyle.paperClip) {
+    const Layout *l = iCascade->findLayout();
+    cairo_rectangle(cc, -l->iOrigin.x, -l->iOrigin.y,
+		    l->iPaperSize.x, l->iPaperSize.y);
+    cairo_clip(cc);
+  }
+
   CairoPainter painter(iCascade, iFonts, cc, iZoom, iStyle.pretty);
   painter.setDimmed(iDimmed);
   // painter.Transform(CanvasTfm());
@@ -439,6 +455,28 @@ void Canvas::drawObjects(cairo_t *cc)
   if (background && iPage->findLayer("BACKGROUND") < 0)
     background->iObject->draw(painter);
 
+  if (iStyle.numberPages) {
+    const StyleSheet::PageNumberStyle *pns =
+      iCascade->findPageNumberStyle();
+    cairo_save(cc);
+    cairo_set_source_rgb(cc, pns->iColor.iRed.toDouble(),
+			 pns->iColor.iGreen.toDouble(),
+			 pns->iColor.iBlue.toDouble());
+    cairo_select_font_face(cc, "sans-serif",
+			   CAIRO_FONT_SLANT_NORMAL,
+			   CAIRO_FONT_WEIGHT_NORMAL);
+    cairo_set_font_size(cc, pns->iFontSize);
+    cairo_move_to(cc, pns->iPos.x, pns->iPos.y);
+    cairo_scale(cc, 1.0, -1.0);
+    char buf[20];
+    if (iPage->countViews() > 0)
+      sprintf(buf, "%d-%d", iPageNumber + 1, iView + 1);
+    else
+      sprintf(buf, "%d", iPageNumber + 1);
+    cairo_show_text(cc, buf);
+    cairo_restore(cc);
+  }
+
   const Text *title = iPage->titleText();
   if (title)
     title->draw(painter);
@@ -602,7 +640,7 @@ Vector Canvas::simpleSnapPos() const
 void Canvas::keyPressEvent(QKeyEvent *ev)
 {
   String s = IpeQ(ev->text());
-  if (iTool && iTool->key(ev->key(), ev->modifiers(), s))
+  if (iTool && iTool->key(ev->key(), ev->modifiers()|iAdditionalModifiers, s))
     ev->accept();
   else
     ev->ignore();
@@ -613,9 +651,10 @@ void Canvas::mousePressEvent(QMouseEvent *ev)
   iGlobalPos = Vector(ev->globalPos().x(), ev->globalPos().y());
   computeFifi(ev->x(), ev->y());
   if (iTool)
-    iTool->mouseButton(ev->button() | ev->modifiers(), true);
+    iTool->mouseButton(ev->button() | ev->modifiers() | iAdditionalModifiers,
+		       true);
   else
-    emit mouseAction(ev->button() | ev->modifiers());
+    emit mouseAction(ev->button() | ev->modifiers() | iAdditionalModifiers);
 }
 
 void Canvas::mouseReleaseEvent(QMouseEvent *ev)
@@ -642,14 +681,14 @@ void Canvas::tabletEvent(QTabletEvent *ev)
 
   switch (ev->type()) {
   case QEvent::TabletPress:
-    if (ev->pointerType() == QTabletEvent::Eraser)
-      return; // not yet implemented
     iGlobalPos = globalPos;
     computeFifi(hiPos.x(), hiPos.y());
-    if (iTool)
-      iTool->mouseButton(Qt::LeftButton, true);
+    if (ev->pointerType() == QTabletEvent::Eraser)
+      emit mouseAction(Qt::MidButton | iAdditionalModifiers);
+    else if (iTool)
+      iTool->mouseButton(Qt::LeftButton | iAdditionalModifiers, true);
     else
-      emit mouseAction(Qt::LeftButton);
+      emit mouseAction(Qt::LeftButton | iAdditionalModifiers);
     break;
   case QEvent::TabletMove:
     if (ev->pressure() > 0.01) {
@@ -712,9 +751,9 @@ void Canvas::paintEvent(QPaintEvent * ev)
 	drawFrame(cc);
       if (iSnap.iGridVisible)
 	drawGrid(cc);
+      drawObjects(cc);
       if (iSnap.iWithAxes)
 	drawAxes(cc);
-      drawObjects(cc);
     }
     cairo_surface_flush(iSurface);
     cairo_destroy(cc);
