@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2009  Otfried Cheong
+    Copyright (C) 1993-2010  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -203,6 +203,7 @@ Dialog::Dialog(lua_State *L0, QWidget *parent) :QDialog(parent)
 {
   L = L0;
   iLuaDialog = LUA_NOREF;
+  iIgnoreEscape = false;
   QGridLayout *lo = new QGridLayout;
   setLayout(lo);
   QShortcut *shortcut = new QShortcut(QKeySequence("Ctrl+Return"), this);
@@ -217,6 +218,13 @@ Dialog::~Dialog()
   luaL_unref(L, LUA_REGISTRYINDEX, iLuaDialog);
 }
 
+void Dialog::keyPressEvent(QKeyEvent *e)
+{
+  if (iIgnoreEscape && e->key() == Qt::Key_Escape)
+    return;
+  QDialog::keyPressEvent(e);
+}
+
 void Dialog::callLua()
 {
   // only call back to Lua during execute()
@@ -471,6 +479,12 @@ static void xmlIndent(QString &text)
 
 int Dialog::set(lua_State *L)
 {
+  QString s = checkstring(L, 2);
+  if (s == "ignore-cancel") {
+    iIgnoreEscape = lua_toboolean(L, 3);
+    return 0;
+  }
+
   int idx = findElement(L, 2);
 
   QLabel *l = qobject_cast<QLabel *>(iElements[idx].widget);
@@ -856,7 +870,7 @@ static int ipeui_getString(lua_State *L)
 				      QLineEdit::Normal,
 				      QString(),
 				      &ok);
-  if (ok && !str.isEmpty()) {
+  if (ok) {
     push_string(L, str);
     return 1;
   } else
